from kazoo.client import KazooClient,KazooState,KeeperState
import json
import time
import logging
import traceback
import sys


class ZkService:
    def __init__(self,config={}):
        #self.zk = config["settings"]
        self.config=config
        zk_config=config
        self.settings = config['settings']
        self.zk = KazooClient(hosts=self.settings.zk.url)
        self.zk.add_listener(onZkStateChanged)

    def register(self):
        try:
            time.sleep(5)
        except Exception as e:
            print(str(e))


        try:
            self.zk.start()
            traefik_path = ('/traefik/apps/%s-%s' % (self.config['APP_NAME'],self.config['uuid']))
            app_root =('%s/%s' % (self.settings.zk.path,self.config['APP_NAME']))
            app_path = ('%s/%s/%s' % (self.settings.zk.path,self.config['APP_NAME'],self.config['uuid']))
            #data = zk.get(settings.zk.path+"/iot-ext/e3da7179-f35b-4200-9732-71cd8f780a2f")
            if not self.zk.exists(traefik_path):
                self.zk.create(traefik_path)#,ephemeral=True)

            api_info = json.dumps(self.config['api_info'].toMap())
            s = str.encode(api_info)
            self.zk.set(traefik_path,s)
            print("--------------uuid:%s--------zk api ---------------------: %s" % (self.config['uuid'],s))
            if not self.zk.exists(app_root):
                self.zk.create(app_root)
            if not self.zk.exists(app_path):
                self.zk.create(app_path)#,ephemeral=True)
            instance_info = json.dumps(self.config['instance_info'].toMap())
            self.zk.set(app_path,str.encode(instance_info))
            print("--------------uuid:%s--------zk was called---------------------" % (self.config['uuid']))
    #        zk.close()
    #        zk.stop()
        except Exception as e:
            traceback.print_exc()
            print('While I was trying to register the node["%s"] to the zk, there was an exception: %s' % (self.config['uuid'],str(e)))
        finally:
            try:
                self.zk.stop()
                print("Close conection with zookeeper")

            except Exception as ee:
                print("Failed to close conection with zookeeper")


def onZkStateChanged(state):
    print("zookeeper client state is changed to %s" % str(state))

    if  state == KazooState.LOST:
        print("zookeeper client state is changed to lost!")
        """
        sys.path.append('../')
        from Settings import Settings
        settings = Settings.fetch_settings(config_url="http://config-server:8801/stat-api/dev", module=app.config['APP_NAME'],
                                profile=app.config['APP_MODE'])

        zk_client = ZkSerivce(settings)

        zk_client.register()
        """
    elif state == KazooState.SUSPENDED:
        print("zookeeper client state is changed to suspended!")
    else:
        print("zookeeper client state is changed to connected!")








import http.client
import json
import time
import ssl
import requests
from utilities.URLUtility import GenericURL
from utilities.DateTimeUtility import DatetimeUtility
from utilities.Utility import Utility
from utilities.GNSSTools import GNSSTools
import logging,traceback

class LBSService:
    def __init__(self,settings):
        import sys
        sys.path.append('../../')
        from utilities.URLUtility import GenericURL
        self.settings = settings
        self.url = GenericURL(settings.lbs.url)

        ssl._create_default_https_context = ssl._create_unverified_context

    def queryByAddress(self,address):
        logging.info('This is a parent class, there is nothing to do!')
        return None


    def query(self,longitude,latitude):
        logging.info('This is a parent class, there is nothing to do!')
        return None


    def queryLocationDesc(self,longitude,latitude):
        retry = 3
        address = None
        if self.validate(longitude,latitude):
            address = self.query(longitude,latitude)
            while address is None and retry > 0:
                address = self.query(longitude,latitude)
                retry = retry -1
                logging.warning("Retry to resolve the address[%f,%f] in %d times" % (latitude,longitude,retry))

        return address

    def validate(self,longitude,latitude):
        if longitude is None or latitude is None or (abs(longitude - 0.0) == 0.0001 and abs(latitude - 0.0) <= 0.0001):
            return False
        else:
            return True

    @staticmethod
    def createLBSService(settings):
        if settings.lbs.provider == 'baidu':
            return BaiduLBSService(settings)
        elif settings.lbs.provider == 'google':
            return GoogleLBSService(settings)
        elif settings.lbs.provider == 'gaode':
            return GaodeLBSService(settings)
        else:
            return None


class GoogleLBSService(LBSService):
    #https://developers.google.com/maps/documentation/geocoding/start
    #https://maps.googleapis.com/maps/api/geocode/json?latlng=40.714224,-73.96145&key=AIzaSyCjLe5tfpxlfNsdo9btD9PeMhRz3xmB5Jc
    def __init__(self,settings):
        LBSService.__init__(self,settings)

    def queryByAddress(self, address):
        logging.warning("To-Do: query address with google map api")
        return None

    def query(self,longitude,latitude):

        resource = ('%s?latlng=%f,%f&output=json&key=%s' % (self.url.path, latitude,longitude, self.url.params['key']))
        try:
            time.sleep(1)
            if self.url.scheme == 'http':
                conn = http.client.HTTPConnection(self.url.host)#httplib.HTTPConnection(host=self.url.host,timeout=30)
            else:
                conn = http.client.HTTPSConnection(self.url.host)#httplib.HTTPSConnection(host=self.url.host,timeout=30)p
            conn.request("GET", resource)
            r1 = conn.getresponse()
            if r1.status != 200:
                logging.warning("Location(%f,%f), result: %s, reason: %s, info: %s)" % (longitude, latitude, str(r1.status), r1.reason, r1.msg))
                return None
            else:
                result = json.loads(r1.read().decode("utf-8"))
                print(result)
                if 'results' in result.keys():
                    address = Address(longitude,latitude)
                    address_components=result['results'][0]['address_components']
                    address.address= result['results'][0]['formatted_address']

                    for c in address_components:
                        if 'administrative_area_level_1' in c['types']:
                            address.province = c['short_name']
                        elif 'administrative_area_level_2' in c['types']:
                            address.city = c['short_name']
                        elif 'country' in c['types']:
                            address.country=c['short_name']
                        elif 'locality' in c['types'] or 'administrative_area_level_3' in c['types']:
                            address.district=c['short_name']

                    return address
                else:
                    return None
        except Exception as e:
            logging.warning("While call %s%s, There is an exception: %s" % (self.url.host,resource,e))
            return None


class BaiduLBSService(LBSService):
    def __init__(self,settings):
        LBSService.__init__(self,settings)

    def query(self,longitude,latitude):
        ll = GNSSTools.gpstobd09(longitude,latitude)
        resource = ('%s?location=%f,%f&output=json&pois=1&ak=%s' % (self.url.path, ll[1], ll[0], self.url.params['ak']))
        conn = None
        result = ""

        r1=None
        try:
            time.sleep(1)
            if self.url.scheme == 'http':
                conn = http.client.HTTPConnection(self.url.host)#httplib.HTTPConnection(host=self.url.host,timeout=30)
            else:
                conn = http.client.HTTPSConnection(self.url.host)#httplib.HTTPSConnection(host=self.url.host,timeout=30)p
            conn.request("GET", resource)
            r1 = conn.getresponse()
            if r1.status != 200:
                logging.warning("Location(%f,%f), result: %s, reason: %s, info: %s)" % (longitude, latitude, str(r1.status), r1.reason, r1.msg))
                return None
            else:
                result = json.loads(r1.read().decode("utf-8"))
                if 'result' in result.keys():
                    # To-Do: 在计算出 [country,province/state,city,ditrict]的数据
                    address = Address(longitude,latitude)
                    address.address= result['result']['formatted_address']
                    address.country = result['result']['addressComponent']['country']
                    address.province = result['result']['addressComponent']['province']
                    address.city = result['result']['addressComponent']['city']
                    address.district = result['result']['addressComponent']['district']
                    address.town = result['result']['addressComponent']['town']
                    return address
                else:
                    return None
        except Exception as e:
            logging.warning("While call %s%s, There is an exception: %s" % (self.url.host,resource,e))
            return None

    def queryByAddress(self, address):
        try:
            payload={
                'ak': self.url.params['ak'],
                'output': 'json',
                'address': address
            }
            url='{}://{}{}'.format(self.url.scheme,self.url.host,self.url.path)
            r=requests.get(url,params=payload)
            ret = r.json()
            if ret['status']  == 0:
                lng = ret['result']['location']['lng']
                lat = ret['result']['location']['lat']
                location=self.query(lng,lat)
                return location
            else:
                location=Address(None,None)
                location.address=address
                return location
        except Exception as e:
            logging.exception(e)
            location=Address(None,None)
            location.address=address
            return location


class GaodeLBSService(LBSService):
    def __init__(self,settings):
        LBSService.__init__(self,settings)

    def queryByAddress(self, address):
        logging.warning("To-Do: query address with gaode map api")
        return None

    def query(self,longitude,latitude):
        resource = ('%s?location=%f,%f&output=json&roadlevel=1&key=%s' % (self.url.path, longitude, latitude, self.url.params['key']))
        conn = None
        result = ""

        r1=None
        try:
            time.sleep(1)
            if self.url.scheme == 'http':
                conn = http.client.HTTPConnection(self.url.host)#httplib.HTTPConnection(host=self.url.host,timeout=30)
            else:
                conn = http.client.HTTPSConnection(self.url.host)#httplib.HTTPSConnection(host=self.url.host,timeout=30)p
            conn.request("GET", resource)
            r1 = conn.getresponse()
            if r1.status != 200:
                logging.warning("Location(%f,%f), result: %s, reason: %s, info: %s)" % (longitude, latitude, str(r1.status), r1.reason, r1.msg))
                return None
            else:
                result = json.loads(r1.read().decode("utf-8"))
                if 'regeocode' in result.keys():
                    # To-Do: 在计算出 [country,province/state,city,ditrict]的数据
                    address = Address(longitude,latitude)
                    address.address= result['regeocode']['formatted_address']
                    address.country = result['regeocode']['addressComponent']['country']
                    address.province = result['regeocode']['addressComponent']['province']
                    address.city = result['regeocode']['addressComponent']['city']
                    address.district = result['regeocode']['addressComponent']['district']
                    address.town = result['regeocode']['addressComponent']['township']
                    return address
                else:
                    return None
        except Exception as e:
            logging.warning("While call %s%s, There is an exception: %s" % (self.url.host,resource,e))
            return None





class Address:
    def __init__(self,longitude,latitude):
        self.country=''
        self.province=''
        self.city=''
        self.district=''
        self.town=''
        self.address=''
        if longitude is not None and latitude is not None:
            self.lnglat={'x':longitude,'y':latitude}
        else:
            self.lnglat=None

    def toMap(self):
        return {'address':self.address,'country':self.country,'province':self.province,'city':self.city,'town':self.town,'district':self.district,'lnglat':self.lnglat}


if __name__ == '__main__':
    import sys

    sys.path.append('../../')
    from Settings import Settings
    settings=Settings.readLocalSettings(path="../../../../test.json")

    #lsbsvc = GaodeLBSService(settings)
    #r = lsbsvc.queryLocationDesc(116.314049758,40.069384452)
    #r = lsbsvc.queryLocationDesc(116.481488,39.990464)
    #print('%s' % (r.toMap()))
    #print('%s' % (json.dumps(r)))
    lbssvc=BaiduLBSService(settings)
    r=lbssvc.queryByAddress("云南省昭通市水富县人民西路36号")
    print('%s' % (r.toMap()))


#encoding=utf-8
import traceback

from pymongo import MongoClient

import logging
operators=['gt','lt','gte','lte','eq','ne','exist','regex','options','not','nor','or','and','in','nin','all','elemMatch','within','near','nearSphere']

class MongoDBService:
    def __init__(self,db,settings=None):
        self.settings = settings
        self.db = db
        self.conn = None

    def __connect(self):
        if self.conn is None:
            self.conn = MongoClient(self.settings.host,self.settings.port)
            db_auth = self.conn.admin
            db_auth.authenticate(self.settings.account,self.settings.password)


    def __close(self):
        if self.conn != None:
            try:
                self.conn.close()
            finally:
                logging.info('Close the connnection with mongodb.')

    def close(self):
        self.__close()

    #向${collection}表新增一条记录${data}
    def insert_one(self,collection,data):
        if data is None:
            return
        self.__connect()
        db=self.conn[self.db]
        tbl=db[collection]
        x = tbl.insert_one(data)
        logging.debug("New a record into {}: {}".format(collection,data))
        return x.inserted_id

    # 向${collection}表新增一条记录${data}
    def insert_many(self, collection, data):
        if data is None:
            return
        self.__connect()
        db = self.conn[self.db]
        tbl = db[collection]
        x = tbl.insert_many(data)
        cnt=len(x.inserted_ids)
        logging.debug("New {} records into {}".format(cnt,collection))
        return cnt




    #根据${query}查找一条记录
    def find_one(self,collection,query):
        self.__connect()
        db = self.conn[self.db]
        tbl = db[collection]

        filter=self.__toMongoOperator(query)
        return tbl.find_one(filter)

    #根据${query}查找多条记录 query: {field: ${filed_filter}},sort: [{field: ${1 | -1}}]
    def find(self,collection,query,sort,page_no,page_size,fields=None):
        self.__connect()
        db = self.conn[self.db]
        tbl = db[collection]
        filter=self.__toMongoOperator(query)
        limit = 0 if page_size is None or page_size <= 0 else page_size
        skip = 0 if page_size is None or page_size <= 0 else (page_no-1)*page_size
        pn = 1 if page_size is None or page_size <= 0 else page_no
        result = None
        if sort is None or len(sort) ==0:
            sort=[('_id', -1)]
        if fields is not None and len(fields) > 0:
            fieldMap = {}

            for field in fields:
                fieldMap[field] = 1
            #result= tbl.find(filter=filter, limit=limit, skip=skip, sort=sort,fields=fieldMap)
            result=tbl.find(filter,fieldMap).sort(sort).skip(skip).limit(limit)
        else:
            #result= tbl.find(filter=filter, limit=limit, skip=skip, sort=sort)
            result=tbl.find(filter).sort(sort).skip(skip).limit(limit)
        total=tbl.find(filter=filter).count()

        return {'total':total,'page_no': pn,'cursor': skip,'limit': limit, 'result':list(result)}

    """
    根据filte计算符合的记录条数
    """
    def count(self,collection,query):
        self.__connect()
        db = self.conn[self.db]
        tbl = db[collection]
        filter=self.__toMongoOperator(query)
        cnt = tbl.find(filter=filter).count()
        return cnt

    """
        更新最先匹配filter的一行记录
        update: {${field}:${value}}
        """

    def upsert(self, collection, query, update):
        self.__connect()
        db = self.conn[self.db]
        tbl = db[collection]
        filter = self.__toMongoOperator(query)
        result = tbl.update_one(filter, {'$set': update},upsert=True)
        return result.matched_countind(filter=filter).count()
        return cnt
    """
    更新最先匹配filter的一行记录
    update: {${field}:${value}}
    """
    def update_one(self,collection,query,update):
        self.__connect()
        db = self.conn[self.db]
        tbl = db[collection]
        filter=self.__toMongoOperator(query)
        result = tbl.update_one(filter,{'$set':update})
        return result.matched_count
    """
    更新最先匹配filter的所有记录
    update: {${field}:${value}}
    """
    def update_many(self,collection,query,update):
        self.__connect()
        db = self.conn[self.db]
        tbl = db[collection]
        filter=self.__toMongoOperator(query)
        result= tbl.update_many(filter,{'$set':update})
        return result.matched_count

    def delete(self,collection,query):
        self.__connect()
        db = self.conn[self.db]
        tbl = db[collection]
        filter=self.__toMongoOperator(query)
        result= tbl.remove(filter,update)
        return result['ok']



    def __fromMathOprator(self,operator):
        if operator=='>':
            return 'gt'
        elif operator == '>=':
            return 'gte'
        elif operator=='<':
            return 'lt'
        elif operator == '<=':
            return 'lte'
        elif operator=='==':
            return 'eq'
        elif operator == '!=':
            return 'ne'
        else:
            return operator


    def __toMongoOperator(self,filters):
        query={}
        for key in filters.keys():
            newKey=self.__fromMathOprator(key)
            if newKey in operators:
                newKey = '$'+key
            v=filters[key]
            if isinstance(v,dict):
                query[newKey]=self.__toMongoOperator(v)
            elif isinstance(v,list):
                vs = []
                for item in v:
                    if isinstance(item,dict):
                        vs.append(self.__toMongoOperator(item))
                    else:
                        vs.append(item)

                query[newKey]=vs
            else:
                query[newKey]=v

        return query











#encoding=utf-8
class APIError(Exception):
    def __init__(self, errCode, errorInfo):
        self.errorinfo = errorInfo
        self.errorCode = errCode

    def __str__(self):
        return '{}(ErrCode={})'.format(self.errorinfo, str(self.errorCode))

class ErrorCodeException(APIError):
    def __init__(self, errCode, errorInfo):
        super().__init__(errCode,errorInfo)

    def __str__(self):
        return 'ErrCode: {}, {}'.format(self.errorinfo, str(self.errorCode))


#coding=utf-8
import json

import time
from datetime import date,datetime,timedelta
# coding=utf-8
import re
import math
import socket


# 工具类
class Utility:
    @staticmethod
    def getTheDayFrom(theDay=None,days=0):
        tdelta = timedelta(days=days)
        current=theDay
        if current is None:
            current = date.today()
        theDay = current +tdelta
        #theTime = datetime.time(0,0,0)
        last = datetime(theDay.year,theDay.month,theDay.day,0,0,0)
        return last

    @staticmethod
    def getTheTimeFrom(theTime=None,days=0,hours=0,minutes=0,seconds=0,ms=0):
        tdelta = timedelta(days=days,hours=hours,minutes=minutes,seconds=seconds,milliseconds=ms)
        current = theTime
        if current is None:
            current = datetime.now()
        theTime = current +tdelta
        return theTime

    @staticmethod
    def getEndTimeFromCurrentDay(theTime=None,hours=0,minutes=0,seconds=59):
        tdelta = timedelta(hours=hours, minutes=minutes, seconds=seconds)
        current = theTime
        if current is None:
            current = datetime.now()
        theTime = current + tdelta
        tdelta = timedelta(hours=hours, minutes=minutes, seconds=seconds+1)
        next = current+tdelta
        if current.day < theTime.day:
            theTime = datetime(current.year,current.month,current.day,23,59,59,0)
            next = Utility.getTheDayFrom(1)
        return (theTime,next)

    # 序列化
    @staticmethod
    def serialize_instance(obj):
        d = {'__classname__': obj.__class__.__name__}
        d.update(vars(obj))
        return d

    # 反序列化
    @staticmethod
    def unserialize_object(d, classes={}):
        clsname = d.pop('__classname__', None)
        if clsname:
            cls = classes[clsname]
            obj = cls()  # Make instance without calling __init__
            dict = d.items()
            for key in dict.keys():
                setattr(obj, key, dict[key])
            return obj
        else:
            return d


    @staticmethod
    def ieee754Int2Float(a, p):
        f = ((-1) ** ((a & 0x80000000) >> 31)) * (2 ** (((a & 0x7f800000) >> 23) - 127)) * (1 + (
                    (a & 0x7fffff) * 1.0 / (
                        2 ** 23)))  # (1 + (math.log((a & 0x7fffff),2)))#((-1)**((a & 0x80000000)>>31))*(2**(((a &0x7f800000)>>23)-127))*(1+((a&0x7fffff)/(2**23*1.0)))
        return round(f, p)

    # ieee754转换
    @staticmethod
    def Float2ieee754Words(f, byte_order):

        words = []
        if (f == 0):
            words.append(0)
            words.append(0)
            return words

        signedBit = 0 if f > 0 else 1

        stepBits = int(math.floor(math.log(abs(f), 2)))
        tailBits = int(((abs(f) / (2 ** stepBits)) - 1) * (2 ** 23)) if stepBits > 0 else int(
            (abs(f) / (2 ** stepBits)) * (2 ** 23))  # int(((f/(2**stepBits))-1) * (2**23))

        dw = ((signedBit << 31) & 0x80000000) | (((stepBits + 127) << 23) & 0x7f800000) | (tailBits & 0x7fffff)
        wh = (dw & 0xffff0000) >> 16
        wl = dw & 0xffff
        if re.search(r'(^cdab$)', byte_order) != None:
            words.append(wl)
            words.append(wh)
        elif re.search(r'(^abcd$)', byte_order) != None:
            words.append(wh)
            words.append(wl)
        elif re.search(r'(^badc$)', byte_order) != None:
            words.append(((wh & 0xff) << 8) | ((wh & 0xff00) >> 8))
            words.append(((wl & 0xff) << 8) | ((wl & 0xff00) >> 8))
        else:  # r'(^dcba$)'
            words.append(((wl & 0xff) << 8) | ((wl & 0xff00) >> 8))
            words.append(((wh & 0xff) << 8) | ((wh & 0xff00) >> 8))
        return words

    @staticmethod
    def ieee754Words2Float(wl, wh, p):
        a = wh << 16 | wl
        # f = ((-1) ** ((a & 0x80000000) >> 31)) * (2 ** (((a & 0x7f800000) >> 23) - 127)) * (1 + ((a & 0x7fffff) / (2 ** 23 * 1.0)))
        # f = ((-1) ** ((a & 0x80000000) >> 31)) * (2 ** (((a & 0x7f800000) >> 23) - 127)) * (1 + (math.log((a & 0x7fffff),2)))
        return Utility.ieee754Int2Float(a, p)

    # int -> words[2]
    @staticmethod
    def toWords(d, byte_order):
        wh = (abs(d) & 0xffff0000) >> 16
        wl = abs(d) & 0xffff
        words = []
        if d < 0:
            wh = wh | 0x8000
        if re.search(r'(^cdab$)', byte_order) != None:
            words.append(wl)
            words.append(wh)
        elif re.search(r'(^abcd$)', byte_order) != None:
            words.append(wh)
            words.append(wl)
        elif re.search(r'(^badc$)', byte_order) != None:
            words.append(((wh & 0xff) << 8) | ((wh & 0xff00) >> 8))
            words.append(((wl & 0xff) << 8) | ((wl & 0xff00) >> 8))
        else:  # r'(^dcba$)'
            words.append(((wl & 0xff) << 8) | ((wl & 0xff00) >> 8))
            words.append(((wh & 0xff) << 8) | ((wh & 0xff00) >> 8))
        return words

    # 		if re.search(r'(^dcba$)|(^cdba$)', byte_order) != None:
    # 			words.append(wh)
    # 			words.append(wl)
    # 		else:
    # 			words.append(wl)
    # 			words.append(wh)
    # 		return words

    # bytes[2] -> word
    @staticmethod
    def toWord(l, h, byte_order, vtype):
        if re.search(r'(^ba($|cd$))|(^dcba$)', byte_order) != None:
            if (re.search('^unsigned ', vtype)):
                t = (((l & 0x7f) << 8) & (((l & 0x8) << 8) & 0xff00) | h)
                return t
            else:
                t = ((l << 8) & 0xff00) | h
                return t
        else:
            if (re.search('^unsigned ', vtype)):
                t = (((h & 0x7f) << 8) & (((h & 0x8) << 8) & 0xff00) | l)
                return t
            else:
                t = ((h << 8) & 0xff00) | l
                return t

    # words[2] -> double words (int)
    @staticmethod
    def toDWord(l0, h0, l1, h1, byte_order, vtype):
        if re.search(r'^dcba$', byte_order) != None:
            if (re.search('^signed ', vtype)):
                s = (((l1 & 0x7f) << 24) & 0xff000000) | ((h1 << 16) & 0xff0000) | ((l0 << 8) & 0xff00) | (h0 & 0xff)
                t = s if (l1 & 0x80) == 0 else (0 - s)
            else:
                t = ((l1 & 0xff) << 24) | (h1 << 16) | (l0 << 8) | h0

        elif re.search(r'^cdab$', byte_order) != None:
            if (re.search('^signed ', vtype)):
                s = (((h1 & 0x7f) << 24) & 0xff000000) | ((l1 << 16) & 0xff0000) | ((h0 << 8) & 0xff00) | (l0 & 0xff)
                t = s if (h1 & 0x80) == 0 else (0 - s)
            else:
                t = ((h1 & 0xff) << 24) | (l1 << 16) | (h0 << 8) | l0

        elif re.search(r'^badc$', byte_order) != None:
            if (re.search('^signed ', vtype)):
                s = (((l0 & 0x7f) << 24) & 0xff000000) | ((h0 << 16) & 0xff0000) | ((l1 << 8) & 0xff00) | (h1 & 0xff)
                t = s if (l0 & 0x80) == 0 else (0 - s)
            else:
                t = ((l0 & 0xff) << 24) | (h0 << 16) | (l1 << 8) | h1

        else:
            if (re.search('^signed ', vtype)):
                s = (((h0 & 0x7f) << 24) & 0xff000000) | ((l0 << 16) & 0xff0000) | ((h1 << 8) & 0xff00) | (l1 & 0xff)
                t = s if (h0 & 0x80) == 0 else (0 - s)
            else:
                t = ((h0 & 0xff) << 24) | (l0 << 16) | (h1 << 8) | l1
        return t

    # ieee754 , byte[4] -> float
    @staticmethod
    def toFloat(l0, h0, l1, h1, byte_order, vtype):
        t = Utility.toDWord(l0, h0, l1, h1, byte_order, vtype)
        return Utility.ieee754Int2Float(t, 2)  # ieee754 converting, precision: default 2


    @staticmethod
    def getLocalIP(host='8.8.8.8',port=808):
        # h=socket.gethostname()
        # ip=socket.gethostbyname(h)
        # return ip
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect((host, port))
            ip = s.getsockname()[0]
        finally:
            s.close()
        return ip


#统计型变量类
class StatisticsVar:
    #id : 变量id
    #current： 当前统计值
    #value： 当前值，通过此值，经过计算，汇总到current
    #timestamp： 本次数据的时间戳
    #last： 上次统计的时间错
    def __init__(self,id=None,current=0,value=None,timestamp=0,last=0):
        self.id = id
        self.current = current
        self.value = value
        self.timestamp = int(timestamp/3600)*3600
        self.endTime = timestamp + 3600 #hourly
        self.last = last

    #json反序列化
    def unwrap(self,str):
        #u = Utilities.utilities()
        s = json.loads(str)
        self.current = s['current']
        self.value = s['value']
        self.timestamp = s['timestamp']
        self.endTime = s['endTime']
        if 'last' in s.keys():
            self.last = s['last']

    #序列化成json
    def wrap(self):
        return json.dumps(self,default=Utility.serialize_instance)

    #统计
    def calc(self,current,value,last):
        self.current = current
        self.count = self.cout +1
        self.last = last
        self.value = value
        #self.timestamp = timestamp

    #清零
    def reset(self,timestamp):
        self.value = 0
        self.timestamp = timestamp
        # 'current' and 'last' could not reset

if __name__ == '__main__':
    ts = int(time.time())
    sv = StatisticsVar()
    sv.current = 1
    sv.value = 232
    sv.timestamp = ts
    str = sv.wrap()
    print(str)
    sv1 = StatisticsVar()
    sv1.unwrap(str)
    print(sv1.value)
    sv.value = 128
    print(str)
    str = sv.wrap()
    print(str)
    sv1 = StatisticsVar()
    sv1.unwrap(str)
    print(sv1.value)
    value ='128'
    v = float("2.2")
    print(v)
    (wl, wh) = Utility.Float2ieee754Words(v, "abcd")
    dw = ((wh & 0xffff) << 16) | (wl & 0xffff)
    bhh = (wh & 0xff00) >> 8
    bhl = wh & 0x00ff
    blh = (wl & 0xff00) >> 8
    bll = wl & 0xff
    print("" + str(bhh) + "-" + str(bhl) + "-" + str(blh) + "-" + str(bll))
    f = Utility.ieee754Int2Float(dw, 2)
    print(str(f))
    s = Utility.ieee754Int2Float(0x42c88000, 3)
    print(str(s))
